@@ -39,9 +39,11 @@
 #include <kmimetyperesolver.h>
 #include <knewmenu.h>
 #include <konqmimedata.h>
-#include <konq_fileitemcapabilities.h>
+#include <kfileitemlistproperties.h>
 #include <konq_operations.h>
+#include <kshell.h>
 #include <kurl.h>
+#include <kurlcombobox.h>
 #include <krun.h>
 
 #include "dolphinmodel.h"
@@ -58,7 +60,7 @@
 #include "filterbar.h"
 #include "kurlnavigator.h"
 #include "viewproperties.h"
-#include "dolphinsettings.h"
+#include "settings/dolphinsettings.h"
 #include "dolphin_generalsettings.h"
 
 DolphinViewContainer::DolphinViewContainer(DolphinMainWindow* mainWindow,
@@ -73,6 +75,7 @@ DolphinViewContainer::DolphinViewContainer(DolphinMainWindow* mainWindow,
     m_view(0),
     m_filterBar(0),
     m_statusBar(0),
+    m_statusBarTimer(0),
     m_dirLister(0),
     m_proxyModel(0)
 {
@@ -87,11 +90,15 @@ DolphinViewContainer::DolphinViewContainer(DolphinMainWindow* mainWindow,
             this, SLOT(dropUrls(const KUrl&, QDropEvent*)));
     connect(m_urlNavigator, SIGNAL(activated()),
             this, SLOT(activate()));
+    connect(m_urlNavigator->editor(), SIGNAL(completionModeChanged(KGlobalSettings::Completion)),
+            this, SLOT(saveUrlCompletionMode(KGlobalSettings::Completion)));
 
     const GeneralSettings* settings = DolphinSettings::instance().generalSettings();
     m_urlNavigator->setUrlEditable(settings->editableUrl());
     m_urlNavigator->setShowFullPath(settings->showFullPath());
     m_urlNavigator->setHomeUrl(settings->homeUrl());
+    KUrlComboBox* editor = m_urlNavigator->editor();
+    editor->setCompletionMode(KGlobalSettings::Completion(settings->urlCompletionMode()));
 
     m_dirLister = new DolphinDirLister();
     m_dirLister->setAutoUpdate(true);
@@ -107,11 +114,11 @@ DolphinViewContainer::DolphinViewContainer(DolphinMainWindow* mainWindow,
     m_proxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
 
     connect(m_dirLister, SIGNAL(clear()),
-            this, SLOT(updateStatusBar()));
+            this, SLOT(delayedStatusBarUpdate()));
     connect(m_dirLister, SIGNAL(percent(int)),
             this, SLOT(updateProgress(int)));
-    connect(m_dirLister, SIGNAL(deleteItem(const KFileItem&)),
-            this, SLOT(updateStatusBar()));
+    connect(m_dirLister, SIGNAL(itemsDeleted(const KFileItemList&)),
+            this, SLOT(delayedStatusBarUpdate()));
     connect(m_dirLister, SIGNAL(completed()),
             this, SLOT(slotDirListerCompleted()));
     connect(m_dirLister, SIGNAL(infoMessage(const QString&)),
@@ -128,8 +135,8 @@ DolphinViewContainer::DolphinViewContainer(DolphinMainWindow* mainWindow,
                              m_proxyModel);
     connect(m_view, SIGNAL(urlChanged(const KUrl&)),
             m_urlNavigator, SLOT(setUrl(const KUrl&)));
-    connect(m_view, SIGNAL(requestContextMenu(KFileItem, const KUrl&)),
-            this, SLOT(openContextMenu(KFileItem, const KUrl&)));
+    connect(m_view, SIGNAL(requestContextMenu(KFileItem, const KUrl&, const QList<QAction*>&)),
+            this, SLOT(openContextMenu(KFileItem, const KUrl&, const QList<QAction*>&)));
     connect(m_view, SIGNAL(contentsMoved(int, int)),
             this, SLOT(saveContentsPos(int, int)));
     connect(m_view, SIGNAL(requestItemInfo(KFileItem)),
@@ -146,11 +153,20 @@ DolphinViewContainer::DolphinViewContainer(DolphinMainWindow* mainWindow,
             this, SLOT(saveRootUrl(const KUrl&)));
     connect(m_view, SIGNAL(redirection(KUrl, KUrl)),
             this, SLOT(redirect(KUrl, KUrl)));
+    connect(m_view, SIGNAL(selectionChanged(const KFileItemList&)),
+            this, SLOT(delayedStatusBarUpdate()));
 
     connect(m_urlNavigator, SIGNAL(urlChanged(const KUrl&)),
             this, SLOT(restoreView(const KUrl&)));
+    connect(m_urlNavigator, SIGNAL(historyChanged()),
+            this, SLOT(slotHistoryChanged()));
 
     m_statusBar = new DolphinStatusBar(this, m_view);
+    m_statusBarTimer = new QTimer(this);
+    m_statusBarTimer->setSingleShot(true);
+    m_statusBarTimer->setInterval(300);
+    connect(m_statusBarTimer, SIGNAL(timeout()),
+            this, SLOT(updateStatusBar()));
 
     m_filterBar = new FilterBar(this);
     m_filterBar->setVisible(settings->filterBar());
@@ -158,6 +174,8 @@ DolphinViewContainer::DolphinViewContainer(DolphinMainWindow* mainWindow,
             this, SLOT(setNameFilter(const QString&)));
     connect(m_filterBar, SIGNAL(closeRequest()),
             this, SLOT(closeFilterBar()));
+    connect(m_view, SIGNAL(urlChanged(const KUrl&)),
+            m_filterBar, SLOT(clear()));
 
     m_topLayout->addWidget(m_urlNavigator);
     m_topLayout->addWidget(m_view);
@@ -178,8 +196,8 @@ DolphinViewContainer::~DolphinViewContainer()
 
 void DolphinViewContainer::setUrl(const KUrl& newUrl)
 {
-    m_urlNavigator->setUrl(newUrl);
     if (newUrl != m_urlNavigator->url()) {
+        m_urlNavigator->setUrl(newUrl);
         // Temporary disable the 'File'->'Create New...' menu until
         // the write permissions can be checked in a fast way at
         // DolphinViewContainer::slotDirListerCompleted().
@@ -236,6 +254,37 @@ bool DolphinViewContainer::isUrlEditable() const
     return m_urlNavigator->isUrlEditable();
 }
 
+void DolphinViewContainer::delayedStatusBarUpdate()
+{
+    // Invoke updateStatusBar() with a small delay. This assures that
+    // when a lot of delayedStatusBarUpdates() are done in a short time,
+    // no bottleneck is given.
+    m_statusBarTimer->start();
+}
+
+void DolphinViewContainer::updateStatusBar()
+{
+    // As the item count information is less important
+    // in comparison with other messages, it should only
+    // be shown if:
+    // - the status bar is empty or
+    // - shows already the item count information or
+    // - shows only a not very important information
+    // - if any progress is given don't show the item count info at all
+    const QString msg(m_statusBar->message());
+    const bool updateStatusBarMsg = (msg.isEmpty()
+                                     || (msg == m_statusBar->defaultText())
+                                     || (m_statusBar->type() == DolphinStatusBar::Information))
+                                    && (m_statusBar->progress() == 100);
+
+    const QString text(m_view->statusBarText());
+    m_statusBar->setDefaultText(text);
+
+    if (updateStatusBarMsg) {
+        m_statusBar->setMessage(text, DolphinStatusBar::Default);
+    }
+}
+
 void DolphinViewContainer::updateProgress(int percent)
 {
     if (!m_showProgress) {
@@ -274,7 +323,7 @@ void DolphinViewContainer::slotDirListerCompleted()
         // it is unclear whether writing is supported
         m_isFolderWritable = true;
     } else {
-        KonqFileItemCapabilities capabilities(KFileItemList() << item);
+        KFileItemListProperties capabilities(KFileItemList() << item);
         m_isFolderWritable = capabilities.supportsWriting();
     }
 
@@ -315,39 +364,18 @@ void DolphinViewContainer::closeFilterBar()
     emit showFilterBarChanged(false);
 }
 
-void DolphinViewContainer::updateStatusBar()
-{
-    // As the item count information is less important
-    // in comparison with other messages, it should only
-    // be shown if:
-    // - the status bar is empty or
-    // - shows already the item count information or
-    // - shows only a not very important information
-    // - if any progress is given don't show the item count info at all
-    const QString msg(m_statusBar->message());
-    const bool updateStatusBarMsg = (msg.isEmpty() ||
-                                     (msg == m_statusBar->defaultText()) ||
-                                     (m_statusBar->type() == DolphinStatusBar::Information)) &&
-                                    (m_statusBar->progress() == 100);
-
-    const QString text(m_view->statusBarText());
-    m_statusBar->setDefaultText(text);
-
-    if (updateStatusBarMsg) {
-        m_statusBar->setMessage(text, DolphinStatusBar::Default);
-    }
-}
-
 void DolphinViewContainer::setNameFilter(const QString& nameFilter)
 {
     m_view->setNameFilter(nameFilter);
-    updateStatusBar();
+    delayedStatusBarUpdate();
 }
 
 void DolphinViewContainer::openContextMenu(const KFileItem& item,
-                                           const KUrl& url)
+                                           const KUrl& url,
+                                           const QList<QAction*>& customActions)
 {
     DolphinContextMenu contextMenu(m_mainWindow, item, url);
+    contextMenu.setCustomActions(customActions);
     contextMenu.open();
 }
 
@@ -392,12 +420,18 @@ void DolphinViewContainer::restoreView(const KUrl& url)
             const QString browser = config.readEntry("BrowserApplication");
             if (!browser.isEmpty()) {
                 app = browser;
+                if (app.startsWith('!')) {
+                    // a literal command has been configured, remove the '!' prefix
+                    app = app.mid(1);
+                }
             }
         } else {
             showErrorMessage(i18nc("@info:status",
                                    "Protocol not supported by Dolphin, Konqueror has been launched"));
         }
-        const QString command = app + ' ' + url.pathOrUrl();
+
+        QString secureUrl = KShell::quoteArg(url.pathOrUrl());
+        const QString command = app + ' ' + secureUrl;
         KRun::runCommand(command, app, app, this);
     } else {
         showErrorMessage(i18nc("@info:status", "Invalid protocol"));
@@ -429,6 +463,24 @@ void DolphinViewContainer::requestFocus()
     m_view->setFocus();
 }
 
+void DolphinViewContainer::saveUrlCompletionMode(KGlobalSettings::Completion completion)
+{
+    DolphinSettings& settings = DolphinSettings::instance();
+    settings.generalSettings()->setUrlCompletionMode(completion);
+    settings.save();
+}
+
+void DolphinViewContainer::slotHistoryChanged()
+{
+    const int index = m_urlNavigator->historyIndex();
+    if (index > 0) {
+        // The "Go Forward" action is enabled. Try to mark
+        // the previous directory as active item:
+        const KUrl url = m_urlNavigator->historyUrl(index - 1);
+        m_view->activateItem(url);
+    }
+}
+
 void DolphinViewContainer::slotItemTriggered(const KFileItem& item)
 {
     KUrl url = item.targetUrl();
